package com.flossom.common.core.mapper;

import com.flossom.common.core.domain.entity.WxClockStatistics;
import org.apache.ibatis.annotations.Param;

import java.util.List;


/**
 * 用户打卡统计信息Mapper接口
 *
 * @author flossom
 * @date 2024-01-30
 */
public interface WxClockStatisticsMapper {
    /**
     * 查询用户打卡统计信息
     *
     * @param id 用户打卡统计信息主键
     * @return 用户打卡统计信息
     */
    public WxClockStatistics selectWxClockStatisticsById(Long id);

    /**
     * 查询用户打卡统计信息列表
     *
     * @param wxClockStatistics 用户打卡统计信息
     * @return 用户打卡统计信息集合
     */
    public List<WxClockStatistics> selectWxClockStatisticsList(WxClockStatistics wxClockStatistics);

    /**
     * 新增用户打卡统计信息
     *
     * @param wxClockStatistics 用户打卡统计信息
     * @return 结果
     */
    public int insertWxClockStatistics(WxClockStatistics wxClockStatistics);

    /**
     * 修改用户打卡统计信息
     *
     * @param wxClockStatistics 用户打卡统计信息
     * @return 结果
     */
    public int updateWxClockStatistics(WxClockStatistics wxClockStatistics);

    /**
     * 删除用户打卡统计信息
     *
     * @param id 用户打卡统计信息主键
     * @return 结果
     */
    public int deleteWxClockStatisticsById(Long id);

    /**
     * 批量删除用户打卡统计信息
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteWxClockStatisticsByIds(Long[] ids);

    List<WxClockStatistics> selectByUserIdAndYearMonth(@Param("userIdList") List<Long> userIdList, @Param("year") int year, @Param("month") int month);
}
